{*******************************************************}
{                                                       }
{       Icall Definitions unit                          }
{       Identify Calls v. 1.0.3                         }
{                                                       }
{       Freeware                                        }
{                                                       }
{       Developer: Stratos Christodoulou                }
{                                                       }
{       Version 1.0.2                                   }
{          Added comments                               }
{       Version 1.0.3                                   }
{          A bug fixed at Msn stringlist (component     }
{          couldn't set Msn.Count = 0                   }
{*******************************************************}

unit iCall;

interface
uses windows,classes,defs,sysutils,extctrls;

Type
//Fires up when a  line is ringing
  TRingEvent = procedure (Caller ,Receiver: string) of object;
//Fires up when a  ringing is stoping
  TEndRingEvent = procedure(const totalSecTime:integer;Reason:string) of object;

//Thread object to catch the events on the isdn controller
  CapiThread = class(TThread)
  private
    { Private declarations }
    procedure alfa;
  protected
    procedure Execute; override;
  end;

  TIcall = class (TComponent)
    private
      capiBuffer:DWORD;
      Fcontrollers:byte;
      controler:DWORD;
      fIsError:Boolean;
      fApplID:DWORD;
      fCapiInstalled:Boolean;
      Fagnostos:String;
      Ftimer:TTImer;
      Factive:boolean;
      Fprofile:TProfileStruct;
      Fversion:String;
      Fmanufacturer:String;
      Fserial:string;
      Flistening:Boolean;
      FonRing:TRingEvent;
      FonEndRing:TEndRingEvent;
      FMsn: TStrings;
      FPLCI:Dword;
      FcapiThread:CapiThread;
      Fstart:DWORD;
      Fstop:Dword;
      procedure SetPrivateCallText(Text:string);
      procedure SetActive(Activate:Boolean);
      procedure CheckCalls(Sender: TObject);
      procedure SetMsn(Value : TStrings);
    public
      { Public declarations }
      constructor Create(AOwner: TComponent); override;
      destructor Destroy; override;
      Function GetProfile:TProfileStruct;
      property ApplId:DWORD read FApplID;
      property Profile:TProfileStruct read Fprofile;
      Property Controllers:byte read Fcontrollers;
      property version:string read Fversion;
      property Manufacturer:string read Fmanufacturer;
      property SerialNumber:string read Fserial;
      property Active:boolean read Factive write SetActive;
      property Listening:boolean read Flistening;
    Published
      property CapiInstalled: boolean read FCapiInstalled;
      property PrivateCallText: String read Fagnostos write SetPrivateCallText;
      property OnRing: TRingEvent read FOnRing write FOnRing;
      property OnEndRing:TendRingEvent read FonEndRing write FonEndRing;
      property Msn : TStrings read FMsn write SetMsn;
  end;

  procedure Register;
  var
    apid:DWORD;
    timetocheck:boolean=false;

implementation

procedure Capithread.alfa;
begin
  timetocheck:=true;
end;

procedure CapiThread.Execute;
begin
  freeonTerminate:=true;
  repeat
    if CAPI_WAIT_FOR_SIGNAL(apid)=0 then
      synchronize(alfa);
  until terminated;
end;

procedure Ticall.SetActive(Activate:Boolean);
var
  L_REQ:TListen_REQ;
begin
  Flistening:=false;
  if fIsError then
    exit;
  if (Activate=Factive) or (fCapiInstalled=false) then
    exit;
  with L_REQ do
  begin
    TotalLength:=sizeof(TListen_REQ);
    ApplID:=fApplID;
    Command:=$5;
    SubCommand:=$80;
    MessageNumber:=1;
    Controller:=controler;
    infoMask:=$0;
    CIPMask:=1;
    CIPMASK2:=0;
    CallingPartyNumber:=0;
    CalledPartyNumber:=0;
  end;
  if CAPI_PUT_MESSAGE(fApplId, @L_REQ) =SUCCESS then
  begin
    Fcapithread.Resume;
    Factive:=activate;
  end
  else
    releaseCapi(FapplID);
    Ftimer.Enabled:=Factive;
end;

procedure TIcall.SetPrivateCallText;
Begin
  if text<>Fagnostos then
    Fagnostos:=text;
end;

Constructor TIcall.Create;
var
  ver:Tversion;
begin
  inherited create(Aowner);
  FMsn := TStringList.Create;
  TstringList(FMsn).Sorted := True;
  controler:=0;
  if CAPI_INSTALLED<>SUCCESS then
    fCapiInstalled:=false
  else
    fCapiInstalled:=True;
  if not fIsError then
  begin
    Fcontrollers:=GetActivesControllers;
    if Fcontrollers>0 then
    begin
      controler:=1;
      Fprofile:=getProfile;
    end
    else
    begin
      controler:=0;
      fIsError:=true;
    end;
  end;
  ver:=GetVersion;
  if ver.CAPIMajor<>0 then
  begin
    fversion:=inttostr(ver.CAPIMajor)+'.'+
    inttostr(ver.CAPIMinor)+'.'+
    inttostr(ver.ManufacturerMajor)+'.'+
    inttostr(ver.ManufacturerMinor);
  end;
  Fmanufacturer:=GetManufacturer;
  Fserial:=GetSerial;
  if componentstate <>[csDesigning] then
  begin
  if fCapiInstalled then
    FApplId:=registerCapi;
  If FapplID=0 then
  begin
    fIsError:=true;
    MessageBox(0,'Can''''t install communication with Capi device!','',mb_OK);
  end
  else
    fisError:=false;
  end;
  Fagnostos:='Private call';
  Factive:=false;
  FcapiThread:=CapiThread.Create(true);
  FTimer := TTimer.Create(self);
  FTimer.Enabled := False;
  FTimer.Interval := 10;
  FTimer.OnTimer := CheckCalls;
  apid:=fapplid;
end;

Destructor TIcall.Destroy;
begin
  if FcapiInstalled then
    ReleaseCapi(fApplID);
  if fcapithread.Terminated=false  then
    Fcapithread.Terminate;
  Inherited destroy;
end;

Function TIcall.GetProfile;
var
  szBuf:array[0..64] of byte;
  dwRet:DWORD;
  pProfile:^TProfileStruct;
begin
  if not FIsError then
  begin
    Dwret:=CAPI_GET_PROFILE(@szBuf,controler);
    if dwRet=SUCCESS then
    begin
      pProfile:=@szBuf;
      result:=pprofile^;
    end;
  end;
end;

procedure TIcall.CheckCalls;
var x: Dword;
    MessageHeader: TMessageHeader;
    ConnectInd: TConnectInd;
    FCaller: string[11];
    FReceiver: string[11];
    FCallerNumber: string;
    msg : string[255];
    l1,l2: integer;
    AlertReq: TAlertREQ;
    ConnectResp: TConnectResp;
    InfoInd: TInfoInd;
    InfoResp: TInfoResp;
    DisConnectInd: TDisconnectInd;
    DisConnectResp: TDisconnectResp;
    ListenReq:TlistenReq;
    Res:string;
begin
  if timetocheck then
    timetocheck:=false
  else
    exit;
  x := CAPI_GET_MESSAGE(fapplId,@CapiBuffer);
  if x = 0 then
  begin
    RTLMoveMemory(@MessageHeader, CapiBuffer, Headerlength);
    Case  MessageHeader.Command of
      CAPI_INFO:
      begin
        If MessageHeader.SubCommand = CAPI_IND then
          RtlMoveMemory(@InfoInd, CapiBuffer + Headerlength, MessageHeader.Totallength - Headerlength);
        With InfoResp do
        begin
          length         := Sizeof(InfoResp);
          ApplId         := fApplId;
          Command        := CAPI_INFO;
          SubCommand     := CAPI_RESP;
          Message_Number := $0;
          PLCI           := infoind.PLCI;
        end;
        CAPI_PUT_MESSAGE(fApplId, @InfoResp);
      end;
      CAPI_LISTEN:
      begin
        Case MessageHeader.SubCommand of
          CAPI_CONF:
          begin
            RTLMoveMemory(@ListenReq, CapiBuffer + HeaderLength,MessageHeader.TotalLength - HeaderLength);
            if listenReq.Info=SUCCESS then
              Flistening:=true
            else
              Flistening:=false;
          end;
        end;
      end;
      CAPI_CONNECT:
      begin
        Case MessageHeader.SubCommand of
          CAPI_IND:
          begin
            Fstart:=GetTickCount;
            RtlMoveMemory(@ConnectInd, CapiBuffer + HeaderLength,MessageHeader.Totallength - HeaderLength);
            Fplci:=connectind.PLCI;
            msg := ConnectInd.Buffer;
            l1 := ord(ConnectInd.Buffer[0]);
            FReceiver := Copy(ConnectInd.Buffer,3,l1-1);
            if FReceiver[1] <> '0' then
              FReceiver := '0' + FReceiver;
            l2 := ord(ConnectInd.Buffer[l1 + 1]);
            if l2 > 0 Then
              if l2 > 2 Then
              begin
                FCaller := Copy(ConnectInd.Buffer,l1+5,l2-2);
                if FCaller[1] <> '0' then
                  FCaller := '0' + FCaller;
                FCallerNumber := FCaller;
              end
              else
                FCallerNumber := FAgnostos;
            if (Assigned(Onring)) and  ((FMsn.IndexOf(FReceiver) > -1) or (FMsn.Count = 0)) then
              OnRing(FcallerNumber,Freceiver);
            with AlertReq do
            begin
              length         := SIZEOF(AlertReq);
              ApplId         := fApplId;
              Command        := CAPI_ALERT;
              SubCommand     := CAPI_REQ;
              Message_Number := $0;
              PLCI           := connectind.PLCI;
              BCI            := $0;
              kpf            := $0;
              uud            := $0;
              fda            := $0;
            end;
            CAPI_PUT_MESSAGE(fApplId,@AlertReq);
            with ConnectResp do
            begin
              Command             := CAPI_CONNECT;
              SubCommand          := CAPI_RESP;
              PLCI                := connectind.PLCI;
              REJECT              := $1;
              BProtocol           := $0;
              ConnectedNumber     := $0;
              ConnectedSubaddress := $0;
              LLC                 := $0;
              AdditionalInfo      := $0;
            end;
            CAPI_PUT_MESSAGE(fApplId, @ConnectResp);
          end;
        end;
      end;
      CAPI_DISCONNECT:
      begin
        Fstop:=GetTickCount;
        if MessageHeader.SubCommand = CAPI_IND then
          RtlMoveMemory(@DisConnectInd, CapiBuffer + Headerlength, MessageHeader.Totallength - Headerlength);
        with DisConnectResp do
        begin
          Length         := SizeOf(DisConnectResp);
          ApplId         := fApplId;
          Command        := CAPI_DISCONNECT;
          SubCommand     := CAPI_RESP;
          Message_Number := $0;
          PLCI           := DisConnectInd.PLCI;
        End;
        CAPI_PUT_MESSAGE(fApplId, @DisConnectResp);
        case disConnectInd.Reason of
          0:res:='No cause available' ;
          $3301:res:='Protocol error,Layer1';
          $3302:res:='Protocol error,Layer2';
          $3303:res:='Protocol error,Layer3';
          $3304:res:='Another application got thant call';
          $3305:res:='Cleared by Call Control Supervisor';
          $349A:res:='External handset opened the line';
          $349f:res:='Line Closed';
          else Res:=inttostr(Disconnectind.reason);
        end;
        if Assigned(OnEndring) then
          OnEndRing(Trunc((Fstop-fstart)/1000),res);
      end;
    end;
  end;
end;

procedure TICall.SetMsn(Value : TStrings);
begin
  if Value.Count = 0 then
  begin                 // start bug fixing
    Fmsn.Clear;         //1.0.3 bug correction
    exit;
  end;
  FMsn.Assign(Value);
end;

procedure Register;
begin
  RegisterComponents('Standard', [TICall]);
end;
end.

